<?php
/*
Plugin Name: Theme Tweaker
Plugin URI: http://www.thulasidas.com/theme-tweaker
Description: Tweak your theme colors (yes, any theme) with no CSS stylesheet editing. To tweak your theme, go to <a href="themes.php?page=theme-tweaker.php"> Appearance (or Design) &rarr; Theme Tweaker</a>, or use the "Settings" link on the right.
Version: 1.30
Author: Manoj Thulasidas
Author URI: http://www.thulasidas.com
*/

/*
Copyright (C) 2008 www.thulasidas.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

if (!class_exists("themeTweaker")) {
  class themeTweaker {
    //constructor
    function themeTweaker() {
    }
    function init() {
      $this->getAdminOptions();
    }
    //Returns an array of admin options
    function getAdminOptions() {
      $mThemeName = get_settings('stylesheet') ;
      $mOldKey = $mThemeName .'_OldColors' ;
      $mNewKey = $mThemeName .'_NewColors' ;
      $mPreKey = $mThemeName .'_Preview' ;
      $mActKey = $mThemeName .'_Activate' ;
      $mCSSKey = $mThemeName .'_Tweaked' ;
      $mTrmKey = $mThemeName .'_Trim' ;
      $mOptions = "themeTweaker" . $mThemeName ;
      $themeTweakerAdminOptions =
        array($mOldKey => array(),
              $mNewKey => array(),
              $mPreKey => true,
              $mActKey => false,
              $mCSSKey => '',
              $mCSSKey => '') ;
      $TTOptions = get_option($mOptions);
      if (!empty($TTOptions)) {
        foreach ($TTOptions as $key => $option)
          $themeTweakerAdminOptions[$key] = $option;
      }
      update_option($mOptions, $themeTweakerAdminOptions);
      return $themeTweakerAdminOptions;
    }

    function tmpColor($hex) {
      $newhex = '%' . substr($hex,-6) . '%' ;
      return $newhex ;
    }

    function negColor($hex) {
      $newhex = dechex(hexdec('ffffff') - hexdec($hex)) ;
      for ($i = strlen($newhex) ; $i < 6 ; $i++) $newhex = '0' . $newhex ;
      $newhex = strtoupper('#' . $newhex) ;
      return $newhex ;
    }

    function greyColor($hex) {
      $r = hexdec($hex[1].$hex[2]) ;
      $g = hexdec($hex[3].$hex[4]) ;
      $b = hexdec($hex[5].$hex[6]) ;
      $y = 0.3*$r + 0.59*$g + 0.11*$b ;
      $yy = dechex($y) ;
      if (strlen($yy) == 1) $yy = '0' . $yy ;
      $newhex = '#' . $yy . $yy . $yy ;
      return $newhex ;
    }

    function sepia($hex) {
      $r = hexdec($hex[1].$hex[2]) ;
      $g = hexdec($hex[3].$hex[4]) ;
      $b = hexdec($hex[5].$hex[6]) ;
      $y = 0.3*$r + 0.59*$g + 0.11*$b ;
      $s = $y/255 ;
      $r = min($y + 77*$s, 255) ;
      $g = min($y + 13*$s, 255) ;
      $b = $y ;

      $r = dechex($r) ;
      if (strlen($r) == 1) $r = '0' . $r ;
      $g = dechex($g) ;
      if (strlen($g) == 1) $g = '0' . $g ;
      $b = dechex($b) ;
      if (strlen($b) == 1) $b = '0' . $b ;
      $newhex = '#' . $r . $g . $b ;
      return $newhex ;
    }

    function random($hex)
    {
      $r = rand(1,255) ;
      $g = rand(1,255) ;
      $b = rand(1,255) ;
      $r = dechex($r) ;
      if (strlen($r) == 1) $r = '0' . $r ;
      $g = dechex($g) ;
      if (strlen($g) == 1) $g = '0' . $g ;
      $b = dechex($b) ;
      if (strlen($b) == 1) $b = '0' . $b ;
      $newhex = '#' . $r . $g . $b ;
      return $newhex ;
    }

    function mapFunc($array, $func) {
      $newarray = array() ;
      foreach ($array as $val) $newarray[] = $this->$func($val) ;
      return $newarray ;
    }

    function shortenColors($str) {
      // replace long hex colors with short ones, if possible
      $hex = array('00', '11', '22', '33', '44', '55', '66', '77',
                   '88', '99', '00', 'AA', 'BB', 'CC', 'EE', 'FF',
                   'aA', 'bB', 'cC', 'eE', 'fF',
                   'Aa', 'Bb', 'Cc', 'Ee', 'Ff',
                   'aa', 'bb', 'cc', 'ee', 'ff') ;

      $colorRE = '/#(' . $hex[0] ;
      foreach ($hex as $key => $val) if ($key > 0) $colorRE .= '|' . $hex[$key] ;
      $colorRE .= '){3}/' ;

      // find long colors
      preg_match_all($colorRE, $str, $colors) ;
      // all the matches are in the first array of the results
      $colors = $colors[0] ;
    }

    function getColors(&$str) {
      // replace the color mnemonics in $str
      $cname = array( 'aqua', 'black', 'blue', 'fuchsia',
                      'gray', 'green', 'lime', 'maroon',
                      'navy', 'olive', 'purple', 'red',
                      'silver', 'teal', 'white', 'yellow') ;

      $hname = array('#00FFFF','#000000','#0000FF','#FF00FF',
                     '#808080','#008000','#00FF00','#800000',
                     '#000080','#808000','#800080','#FF0000',
                     '#C0C0C0','#008080','#FFFFFF','#FFFF00') ;
      foreach ($cname as $key => $val){
        $reg[$key] = '/\b' ;
        for ($i=0 ; $i < strlen($val) ; $i++) {
          $reg[$key] .= '[' . $val[$i] . strtoupper($val[$i]) . ']' ;
        }
        $reg[$key] .= '\b/' ;
      }
      $str = preg_replace($reg, $hname, $str) ;

      // Now find the hex color names
      $colorRE = "/#[a-fA-F0-9]{6}|#[a-fA-F0-9]{3}\b/" ;
      preg_match_all($colorRE, $str, $colors) ;

      // all the matches are in the first array of the results
      $colors = array_unique($colors[0]) ;

      // search and replace non-standard names to std long names
      $search = array() ;
      $replace = array() ;
      $matches = array() ;
      foreach ($colors as $val) {
        if (strlen($val) == 4) {
          $search[] = '/' . $val . '\b/' ;
          // short color - make it long
          $tmp = '#' ;
          for ($i=1 ; $i <= 3 ; $i++) {
            $tmp .= $val[$i] . $val[$i] ;
          }
          $val = $tmp ;
          $val = strtoupper($val) ;
          $replace[] = $val ;
        }
        $matches[] = strtoupper($val) ;
      }
      $colors = $matches ;

      // this cannot be a str_replace because #000010 -> #000000010!
      $str = preg_replace($search, $replace, $str) ;

      // make unique and sort
      $colors = array_unique($colors) ;
      sort($colors) ;

      return $colors ;
    }

    // make a color table
    function makeTable($colors0, $colors1) {
      $table = '<table align="center" border="0" padding="0">' . "\n" ;
      $table .= '<tr align="center" valign="middle"><td><b>Old Colors</b></td>' . "\n" .
        '<td><b>Tweaked Colors</b><br />Click to Modify</td></tr>' ;
      foreach ($colors0 as $key => $val) {
        $newcol = $colors1[$key] ;
        $name = substr($val,-6) ;
        $nopicker = '<input readonly class="color {picker:false}" ' .
          'style="border:0px solid;" value="' . $val . '" title="Original Color [read only]">' ;
        $picker = '<input style="border:0px solid;" class="color {hash:true,caps:true,' .
          'pickerFaceColor:\'transparent\',pickerFace:3,pickerBorder:0,' .
          'pickerInsetColor:\'black\'}" onchange="document.getElementById(\'td_' .
          $name . '\').bgColor = \'#\'+this.color" value="' .
          $newcol . '" name="in_' . $name . '" id="in_' . $name .
          '" title="Tweaked Color [Click to pick, or Type in RRGGBB]">' ;

        $table .=  '<tr><td bgcolor="' . $val . '">' . $nopicker . '</td>' . "\n" .
          '<td bgcolor="' . $newcol . '" id="td_' . $name . '">' .
          $picker . '</td></tr>' . "\n" ;
      }
      $table .= '</table>' ;
      return $table ;
    }

    function initNewColors($colors, $newcol) {
      $js = '' ;
      foreach ($newcol as $key => $val) {
        $name = substr($colors[$key],-6) ;
        $js .= 'document.getElementById(\'td_' . $name . '\').bgColor=\'' . $val . '\';' .
          'document.getElementById(\'in_' . $name . '\').color.fromString(\'' . $val . '\');' ;
      }
      return $js ;
    }

    function trimCSS1($css){
      // beginning part
      $reg = '[\s]*[a-zA-Z0-9\.# -_:@]+[\s]*\{' ;
      ereg($reg, $css, $start) ;
      $trim = $start[0] ;
      $reg = '/[^;^{]*#[a-fA-F0-9]{6}[^;\b]*;/' ;
      preg_match_all($reg, $css, $clines) ;
      $clines = $clines[0] ;
      if (count($clines) > 0) {
        foreach ($clines as $val) $trim .= $val ;
        $trim .= "}\n" ;
      }
      else $trim = '' ;
      return $trim ;
    }

    function trimCSS($stylestr) {
      // Try to trim the CSS string so that only the modified lines are kept.
      // Need to test it thoroughly.

      // look for CSS blocks
      $reg = '/[\s]*[a-zA-Z0-9\.# -_:@]+[\s]*\{[^\}]+\}/' ;
      preg_match_all($reg, $stylestr, $css) ;
      $css = $css[0] ;
      $output = '' ;
      // loop over the css blocks and find the styles with colors
      foreach ($css as $val) $output .= $this->trimCSS1($val) ;
      return $output ;
    }

    function makeButtons($colors0, $colors1) {
      $mThemeName = get_settings('stylesheet') ;
      $table = '' ;
      $table .= '<table align="center">' . "\n" . '<tr><td align="center">' ;

      // Reset
      $table .= '</td></tr>' . "\n" . '<tr><td align="center">' ;
      $table .= '<input type="button" style="width:100%;" name="reset" value="Reset Colors" ' ;
      $table .= 'title="Reset the colors to the original colors of ' ;
      $table .= $mThemeName . '" ' ;
      $table .= 'onclick=" ' ;
      $table .= $this->initNewColors($colors0, $colors0) ;
      $table .= '" />' ;

      // invert colors
      $table .= '</td></tr>' . "\n" . '<tr><td align="center">' ;
      $newcol = $this->mapFunc($colors0, 'negColor') ;
      $table .= '<input type="button" style="width:100%;" name="negative" value="Invert Colors" ' ;
      $table .= 'title="Color negatives of the original colors in ' ;
      $table .= $mThemeName. '" ' ;
      $table .= 'onclick=" ' ;

      $table .= $this->initNewColors($colors0, $newcol) ;
      $table .= '" />' ;

      // grey scale
      $table .= '</td></tr>' . "\n" . '<tr><td align="center">' ;
      $newcol = $this->mapFunc($colors0, 'greyColor') ;
      $table .= '<input type="button" style="width:100%;" name="grey" value="Black & White" ' ;
      $table .= 'title="Desaturate to grey scales of the original colors of ' ;
      $table .= $mThemeName. '" ' ;
      $table .= 'onclick=" ' ;
      $table .= $this->initNewColors($colors0, $newcol) ;
      $table .= '" />' ;

      // grey scale negative
      $table .= '</td></tr>' . "\n" . '<tr><td align="center">' ;
      $newcol = $this->mapFunc($colors0, 'negColor') ;
      $newcol = $this->mapFunc($newcol, 'greyColor') ;
      $table .= '<input type="button" style="width:100%;" name="greyneg" value="B&W Negative" ' ;
      $table .= 'title="Negative of the desaturated colors to the original colors of ' ;
      $table .= $mThemeName. '" ' ;
      $table .= ' onclick=" ' ;
      $table .= $this->initNewColors($colors0, $newcol) ;
      $table .= '" />' ;

      // sepia
      $table .= '</td></tr>' . "\n" . '<tr><td align="center">' ;
      $newcol = $this->mapFunc($colors0, 'sepia') ;
      $table .= '<input type="button" style="width:100%;" name="sepia" value="Sepia Effect" ' ;
      $table .= 'title="Generate sepia colours out of the original colors of ' ;
      $table .= $mThemeName. '" ' ;
      $table .= 'onclick=" ' ;
      $table .= $this->initNewColors($colors0, $newcol) ;
      $table .= '" />' ;

      // random colors
      $table .= '</td></tr>' . "\n" . '<tr><td align="center">' ;
      $newcol = $this->mapFunc($colors0, 'random') ;
      $table .= '<input type="button" style="width:100%;" name="random" value="Random Colors" ' ;
      $table .= 'title="Generate random colors while keeping the styles of ' ;
      $table .= $mThemeName. '" ' ;
      $table .= 'onclick=" ' ;
      $table .= $this->initNewColors($colors0, $newcol) ;
      $table .= '" />' ;

      // refresh the page
      $table .= '</td></tr>' . "\n" . '<tr><td align="center">' ;
      $table .= '<input type="button" style="width:100%;" name="refresh" value="Refresh" ' ;
      $table .= 'title="Reload the page (for new set of random colors) " ' ;
      $table .= 'onclick=" window.location.reload( true );" />' ;

      // table closing tags
      $table .= '</td></tr>' . "\n" . '</table>' ;
      return $table ;
    }

    //Prints out the admin page
    function printAdminPage() {
      $mThemeName = get_settings('stylesheet') ;
      $mOldKey = $mThemeName .'_OldColors' ;
      $mNewKey = $mThemeName .'_NewColors' ;
      $mPreKey = $mThemeName .'_Preview' ;
      $mActKey = $mThemeName .'_Activate' ;
      $mCSSKey = $mThemeName .'_Tweaked' ;
      $mTrmKey = $mThemeName .'_Trim' ;
      $TTOptions = $this->getAdminOptions();
      $msg = '';
      // grab the theme stylesheet and print it here
      $stylefile = get_theme_root() . '/' . $mThemeName . '/style.css' ;

      // $stylecontent = file_get_contents($stylefile) ;
      $stylecontent = file_get_contents($stylefile) ;
      $colors0 = $this->getColors($stylecontent) ;

      // if the theme colors haven't changed, use the new colors from DB
      // else init them to the original colors
      if ($colors0 == $TTOptions[$mOldKey]) $colors1 = $TTOptions[$mNewKey] ;
      else $colors1 = $colors0 ;

      if (isset($_POST['update_themeTweakerSettings']) ||
          isset($_POST['saveCSS'])) {
        // loop over the new color fields to get colors1
        foreach ($colors0 as $key => $val) {
          $name = 'in_' . substr($val,-6) ;
          if (isset($_POST[$name])) $colors1[$key] = $_POST[$name] ;
          else $colors1[$key] = '#FFFFFF' ;
        }
        // check activate and preview buttons
        if (isset($_POST['preview'])){
          // echo 'Will Preview ' ;
          $TTOptions[$mPreKey] = true ;
        }
        else {
          // echo 'Will NOT Preview' ;
          $TTOptions[$mPreKey] = false ;
        }
        if (isset($_POST['activate'])){
          // echo 'New scheme is ACTIVE' ;
          $TTOptions[$mActKey] = true ;
        }
        else {
          // echo 'New scheme INACTIVE' ;
          $TTOptions[$mActKey] = false ;
        }

        // need to replace in two steps, just incase colors0 and colors1 have overlaps
        $tmpcols = $this->mapFunc($colors0, 'tmpColor') ;

        // generate the new style
        $styletmp = str_replace($colors0, $tmpcols, $stylecontent) ;
        $stylestr = str_replace($tmpcols, $colors1, $styletmp) ;
        $TTOptions[$mOldKey] = $colors0 ;
        $TTOptions[$mNewKey] = $colors1 ;
        $TTOptions[$mCSSKey] = $stylestr ;
        $trimstr = $this->trimCSS($stylestr) ;
        $TTOptions[$mTrmKey] = $trimstr ;
        $mOptions = "themeTweaker" . $mThemeName ;
        update_option($mOptions, $TTOptions);
        if (isset($_POST['saveCSS'])){
          $styleurl = get_option('siteurl') . '/' . "/wp-admin/themes.php?page=theme-tweaker.php&save" ;
?>
<iframe id="saveCSS" src="about:blank" style="visibility:hidden;border:none;height:1em;width:1px;"></iframe>
<script type="text/javascript">
var fram = document.getElementById("saveCSS");
<?php echo 'fram.src = "'  . $styleurl .'"' ; ?>
</script>
<div class="updated"><p><strong><?php _e('Updated Settings and Generated the CSS Stylefile for your theme "' . $mThemeName . '."', "themeTweaker");?></strong></p></div>
<?php
          return ;
        }

?>
<div class="updated"><p><strong><?php _e('Updated Settings for your theme "' . $mThemeName . '."', "themeTweaker");?></strong></p></div>

<?php
} ?>
<?php echo '<script type="text/javascript" src="'. get_option('siteurl') . '/' . PLUGINDIR . '/' .  basename(dirname(__FILE__)) . '/jscolor/jscolor.js"></script>' ?>
<div class=wrap>
<form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>">
<h2>Theme Tweaker</h2>

<table class="form-table">
<th scope="row"><h3>Instructions</h3></th>
<tr valign="top">
<td>
<ul style="padding-left:10px;list-style-type:circle; list-style-position:inside;" >
<?php echo $msg; ?>
<li>
The color scheme of your current theme <?php echo $mThemeName ; ?> is shown in the table below as the first column under "Old Colors".
</li>
<li>
The new color scheme is in the second column under "Tweaked Colors (Click to modify)." Click on the new colors to modify them. You will get a color picker, or you can type in the new color code as #RRGGBB.
</li>
<li>
Launch the new color scheme in the "preview mode" by checking the preview box. (Preview means only admins can see the changes.)
</li>
<li>
Or choose to "Activate" the new scheme (for everybody) by checking that box.
</li>
<li>
Once ready, click on the "Save Changes" button to save the changes. Note that you will see the changes (in preview or activate mode) only after saving.
</li>
<li>
You can download the tweaked theme style sheet by clicking on the "Download Stylesheet" button. It saves the changes and then downloads a style.css file that you can upload to your blog server theme directory to make your changes permanent.
</li>
<li>
<b>Theme Tweaker </b> will remember your saved color schemes for any number of themes.
</li>
</ul>
</td>
</tr>
</table>

<table class="form-table">
<th scope="row"><h3>Tweak the Colors Found in <?php echo $mThemeName ; ?></h3></th>
<tr></tr>
</table>
<table>
<tr valign="top">
<td>
<table>
<tr align="center" valign="middle">
<td width="350">
<?php echo $this->makeTable($colors0, $colors1) ?>
</td>
<td width="200">
<?php echo $this->makeButtons($colors0, $colors1) ?>
</td>
</tr>
</table>
</table>

<table class="form-table" >
<th scope="row"><h3>Preview or Activate the Tweaked <?php echo $mThemeName ; ?>?</h3></th>
<tr></tr>
</table>
<table class="form-table" >
<tr><td>
<input type="checkbox" name="preview" value="preview" <?php if ($TTOptions[$mPreKey]) echo 'checked'; ?> /> &nbsp;&nbsp; Preview the new color scheme (Only Administrators will see the changes)<br />
<input type="checkbox" name="activate" value="activate" <?php if ($TTOptions[$mActKey]) echo 'checked'; ?> /> &nbsp;&nbsp; Activate the new color scheme (All users will see the changes)<br />
</td>
</tr>
</table>

<div class="submit">
<input type="submit" name="update_themeTweakerSettings" value="<?php _e('Save Changes', 'themeTweaker') ?>" title="Save your options" />
<input type="submit" name="saveCSS" title="Download the tweaked stylesheet to your computer" value="<?php _e('Download Stylesheet', 'themeTweaker') ?>" />
</div>
</form>
<br />
<hr />

<table class="form-table" >
<th scope="row"><h3>Support Theme Tweaker</h3></th>
<tr><td>
<ul style="padding-left:10px;list-style-type:circle; list-style-position:inside;" >
<li>
If you would like support my efforts, please visit (and link-back to) my blog - <a href="http://www.Thulasidas.com" target="_blank">Unreal Blog</a>.
</li>
<li>
<?php echo 'If you like the content and style of my blog, <a href="http://www.Thulasidas.com/about/about-my-book" target="_blank">consider buying my book</a>. In other words, lend me your eyes, but don\'t give me any money unless you like what you see. <img src="' . get_option( 'siteurl' ) . '/wp-includes/images/smilies/icon_smile.gif" alt=":-)"  class="wp-smiley" style="vertical-align:-30%;"; />' ; ?>
</li>
<li>
<b>Theme Tweaker</b> adds a tiny link text at the bottom of your pages like this:<br />
<center><font size="-3"><a href="http://www.Thulasidas.com/theme-tweaker" target="_blank" title="The simplest way to tweak your theme colors!"> Theme Tweaker</a> by <a href="http://www.Thulasidas.com/" target="_blank" title="Unreal Blog proudly brings you Theme Tweaker">Unreal</a></font></center>
Consider leaving it unchanged.
</li>
<li>
If you must delete it, please edit <?php echo get_option('siteurl') . '/' . PLUGINDIR . '/' . basename(dirname(__FILE__)) . '/' . basename(__FILE__) ; ?> and remove all reference to $unreal. (I know I could've made it an option here. But I shouldn't make it too easy, should I?) <?php echo '<img src="' . get_option( 'siteurl' ) . '/wp-includes/images/smilies/icon_smile.gif" alt=":-)"  class="wp-smiley" align="absmiddle" />' ; ?>
</li>
<li>
Check out my other plugin efforts:
<ul style="margin-left:0px; padding-left:30px;list-style-type:square; list-style-position:inside;" >
<li>
<a href="http://www.Thulasidas.com/adsense" target="_blank" title="The simplest way to put AdSense to work for you!"><b>Easy AdSenser</b></a>: The simplest way to put AdSense to work for you. It puts ads like this into your <em>existing</em> posts.
<table bgColor="#EAF3FA">
<tr>
<td align="center" style="border-bottom-width: 0px;">
<?php echo '<script type="text/javascript"><!--
google_ad_client = "pub-1213643583738263";
/* 234x60 ezAdsense, created 11/25/08 */
google_ad_slot = "8050392339";
google_ad_width = 234;
google_ad_height = 60;
//-->
</script>
<script type="text/javascript"
src="http://pagead2.googlesyndication.com/pagead/show_ads.js">
</script>' ?><br />
<font size="-3"><a href="http://wordpress.org/extend/plugins/easy-adsenser/" target="_blank" title="The simplest way to put AdSense to work for you!">Easy AdSenser</a> by <a href="http://www.Thulasidas.com/" target="_blank" title="Unreal Blog proudly brings you Easy AdSenser">Unreal</a></font></td>
</tr>
</table>
</li>
<li>
<a href="http://wordpress.org/extend/plugins/easy-latex/" target="_blank" title="LaTeX in your posts"><b> Easy LaTeX</b></a>: To translate LaTeX formulas like this [math](a+b)^2 = a^2 + b^2 + 2ab[/math] into this: <br/><img src="http://l.wordpress.com/latex.php?latex=(a%2bb)^2%20=%20a^2%20%2b%20b^2%20%2b%202ab&bg=FFFFFF&s=1" style="vertical-align:-70%;" />
</li>
</ul>
</li>
</ul>
</td>
</tr>
</table>

<table class="form-table" >
<th scope="row"><h3>Credit</h3></th>
<tr><td>
<ul style="padding-left:10px;list-style-type:circle; list-style-position:inside;" >
<li>
<b>Theme Tweaker</b> uses the excellent Javascript color picker by <a href="http://jscolor.com" target="_blank" title="Javascript color picker"> JScolor</a>.
</li>
</ul>
</td>
</tr>
</table>

<table class="form-table" >
<th scope="row"><h3>Future Plans</h3></th>
<tr><td>
<ul style="padding-left:10px;list-style-type:circle; list-style-position:inside;" >
<li>
Use Wordpress coding examples for better security and user-level controls.
</li>
<li>
Provide graceful tolerance to lack of Javascript.
</li>
<li>
Give other tweaking shortcuts: Sepia, Change Contrast or Brightness.
</li>
<li>
Handle RGB(r,g,b) or fancier color mnemonic specification.
</li>
<li>
Let viewers of your blog change colors?
</li>
<li>
Save multiple color schemes for the save theme?
</li>
<li>
You tell me!
</li>
</ul>
</td>
</tr>
</table>


</div>

<?php

  }//End function printAdminPage()

    function plugin_action($links, $file) {
      if ($file == plugin_basename(dirname(__FILE__).'/theme-tweaker.php')){
        $settings_link = "<a href='themes.php?page=theme-tweaker.php'>" .
          __('Settings', 'theme-tweaker') . "</a>";
        array_unshift( $links, $settings_link );
      }
      return $links;
    }

    function footer_action(){
      $unreal = '<div align="center"><font size="-3">' .
        '<a href="http://wordpress.org/extend/plugins/theme-tweaker/" ' .
        'target="_blank" title="The simplest way to tweak your theme colors!"> ' .
        'Theme Tweaker</a> by <a href="http://www.Thulasidas.com/" ' .
        'target="_blank" title="Unreal Blog proudly brings you Theme Tweaker">' .
        'Unreal</a></font></div>';
      echo $unreal ;
    }

    function head_action(){
      // if it is a theme preview (from theme switcher), don't insert my style string
      if (isset($_GET['preview'])) return ;
      $mThemeName = get_settings('stylesheet') ;
      $mTrmKey = $mThemeName .'_Trim' ;
      $mPreKey = $mThemeName .'_Preview' ;
      $mActKey = $mThemeName .'_Activate' ;
      $TTOptions = $this->getAdminOptions();
      $stylestr = $TTOptions[$mTrmKey] ;
      if ($TTOptions[$mActKey] || ($TTOptions[$mPreKey] && current_user_can('switch_themes')))
        echo '<style type="text/css"> ' . $stylestr . ' </style>' ;
    }

    function saveCSS() {
      header('Content-Disposition: attachment; filename="style.css"');
      header("Content-Transfer-Encoding: ascii");
      header('Expires: 0');
      header('Pragma: no-cache');
      $TTOptions = $this->getAdminOptions();
      $mThemeName = get_settings('stylesheet') ;
      $mCSSKey = $mThemeName .'_Tweaked' ;
      $stylestr = $TTOptions[$mCSSKey] ;
      ob_start() ;
      print $stylestr ;
      ob_end_flush() ;
      die() ;
    }
  }
} //End Class themeTweaker

if (class_exists("themeTweaker")) {
  $thmTwk = new themeTweaker();
  if (isset($thmTwk)) {
    //Initialize the admin panel
    if (!function_exists("themeTwk_ap")) {
      function themeTwk_ap() {
        global $thmTwk ;
        if (function_exists('add_theme_page')) {
          add_theme_page('Theme Tweaker', 'Theme Tweaker', 9,
                         basename(__FILE__), array(&$thmTwk, 'printAdminPage'));
        }
      }
    }

    if (isset($_GET['save'] ))
      add_action('init', array(&$thmTwk, 'saveCSS'));
    else
      remove_action('init', array(&$thmTwk, 'saveCSS'));

    add_action('admin_menu', 'themeTwk_ap');
    add_action('activate_' . basename(dirname(__FILE__)) . '/' . basename(__FILE__),
               array(&$thmTwk, 'init'));
    add_filter('plugin_action_links', array($thmTwk, 'plugin_action'), -10, 2);
    add_action('wp_head', array($thmTwk, 'head_action'));
    add_action('wp_footer', array($thmTwk, 'footer_action'));
  }
}


/*
To Do
1. Add a button to clean up the DB (in the unlikely event that the user wants to deactivate)
*/

?>
